'':::::::[ Dual FIR Filter ]:::::::::::::::::::::::::::::::::::::::::::::::::::

{{{

******************************************
*   Dual FIR Filter Propeller Object     *
*  generated by www.phipi.com/fir2pasm   *
*  from the parameters given in the DAT  *
*          section of this file.         *
* (c) Copyright 2011 Bueno Systems, Inc. *
*    See end of file for terms of use.   *
******************************************

Created: Sun Aug  7 19:00:49 2011 UTC

}}

''=======[ Introduction ]======================================================

{{ This object provides a finite inpulse response (FIR) filter capable of filtering
   two channels of input at a rate set by the user.
''   
   `Usage:
''     
     1. Call `start with an argument of 0 or the address of a two-long array.
''     
     2. `start stops a prior instance of the same object, if one is running.
''     
     3. Once started, the address provided to `start or the one provided by `start
        upon return will point to a "busy" long and, at the next location, a "data" 
        long.
''        
     From `Spin:
''     
     4. Call `filtera, `filterb, or `filterab with the input datum as its single
        argument. It will return the next output(s) from the filter.
''        
     From `PASM:
''        
     4. To issue the next data for filtering, write them to the "data" address(es),
        then write 1, 2, or 3 to the "busy" address. When the "busy" long
        reverts back to zero, the filter output will be available at the "data"
        address(es).
}}

''=======[ Constants ]=========================================================   

CON

  SIZE          = 54
  BITS          = 18

''=======[ Instance Variables ]================================================

VAR

  long  busy, value[2]
  word  busyaddr
  byte  cogno

''=======[ Public Spin Methods ]===============================================

PUB start(busy_addr)

  {{ Start the FIR filter in a new cog.
  ''
  '' `Parameters:
  ''
  ''     `busy_addr: Either 0 or the address of a three-long communication array.
  ''
  '' `Return: The address provided by `busy_addr or the address of the internal
  ''      array if 'busy_addr == 0 if a cog was started; or False (0) if no cog
  ''      was available.
  ''
  '' `Example: fir.start(0)
  ''
  ''     Start the FIR filter object using its internal communication array.
  }}
  
  stop
  if (busy_addr)
    busyaddr := busy_addr
  else
    busyaddr := @busy
  cogno := cognew(@fir_filter, busyaddr)
  return busyaddr & (++cogno <> 0)
  
PUB stop

  {{ Stop the FIR filter cog if one was started.
  ''
  '' `Example: fir.stop
  ''
  ''     Stop the FIR filter cog.
  }}

  if (cogno)
    cogstop(cogno~ - 1)  
  
PUB filterab(yy)

  {{ Return the address of the next FIR-filtered values from sample
     `yy in the data stream using both filters A and B.
  ''
  '' `Parameters:
  ''
  ''     `yy: The next input sample to filter.
  ''
  '' `Return: The address of the A and B filter output values (longs).
  }}

  long[busyaddr][1] := yy
  long[busyaddr] := 3
  repeat while long[busyaddr]
  return busyaddr + 4
  
PUB filtera(yy)

  {{ Return the next FIR-filtered value from sample `yy in the data stream
     using filter A.
  ''
  '' `Parameters:
  ''
  ''     `yy: The next input sample to filter.
  ''
  '' `Return: The next output from the filter.
  }}

  long[busyaddr][1] := yy
  long[busyaddr] := 1
  repeat while long[busyaddr]
  return long[busyaddr][1]

PUB filterb(yy)

  {{ Return the next FIR-filtered value from sample `yy in the data stream
     using filter B.
  ''
  '' `Parameters:
  ''
  ''     `yy: The next input sample to filter.
  ''
  '' `Return: The next output from the filter.
  }}

  long[busyaddr][1] := yy
  long[busyaddr] := 2
  repeat while long[busyaddr]
  return long[busyaddr][2]

''=======[ PASM Code ]============================================================

{{ This is the code that does the actual filtering work, based upon the parameters
   and coefficients shown below in the source comments.
}}

DAT

'==========================================================================
'Parameters Given
'
'  54 coefficients:
'
'    +4.86550357e-04
'    -1.90150514e-18
'    -3.09895846e-04
'    -1.36088348e-17
'    -2.46240360e-03
'    -3.70616120e-17
'    -6.27828009e-03
'    -7.09139957e-18
'    -1.17771962e-02
'    -2.17097558e-17
'    -1.85709556e-02
'    -3.70845112e-18
'    -2.57832159e-02
'    +9.97762171e-18
'    -3.20112468e-02
'    +4.44885291e-18
'    -3.52903343e-02
'    +1.13751997e-16
'    -3.29206624e-02
'    +1.21585760e-16
'    -2.06864102e-02
'    +2.65482707e-16
'    +1.05834127e-02
'    +2.30557278e-16
'    +9.60337404e-02
'    +9.37440774e-17
'    +8.26419921e-01
'    +7.06377074e-17
'    -3.59972563e-01
'    -0.00000000e+00
'    -1.71855033e-01
'    +7.48794537e-18
'    -1.07442668e-01
'    -4.91903728e-18
'    -6.85955289e-02
'    +1.04926443e-17
'    -4.10053682e-02
'    -2.57626475e-17
'    -2.09374081e-02
'    -5.38549354e-17
'    -6.99327816e-03
'    -6.96305638e-17
'    +1.74716680e-03
'    -6.86471864e-17
'    +6.23595717e-03
'    -5.59730818e-17
'    +7.54229394e-03
'    -3.59543389e-17
'    +6.77689211e-03
'    -7.85818939e-18
'    +4.96627755e-03
'    -2.63107929e-18
'    +2.93753774e-03
'    +2.54533798e-18
'
'  Stated gain:  1.0000000
'  Desired gain: 1.0000000
'  Precision:    18 bits
'  Channels:     Dual Reversed
'
'Resulting PASM instructions (not including overhead):
'
'  h*x computation:    151
'  Each y computation: 80
'
'==========================================================================

              org      0
fir_filter    mov      xy_addr,par              'Initialize the hub address for the data exchange.
              add      xy_addr,#4
              mov      yb_addr,par
              add      yb_addr,#8

'Further user initialization code can be added here.

main_lp                                         'DO NOT MODIFY.

'The next line can be altered to acquire the next input sample in another way.
'On return, sel indicates which channel(s) to process: bit 0 for a; bit 1 for b.

              call      #get_x                  'Read the next input.

'Computes the h * x values for the current x.

              mov       h+13,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              mov       h+12,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+13,x                  'DO NOT MODIFY.
              mov       h+11,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+12,x                  'DO NOT MODIFY.
              mov       h+10,x                  'DO NOT MODIFY.
              mov       h+14,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              add       h+13,x                  'DO NOT MODIFY.
              mov       h+9,x                   'DO NOT MODIFY.
              sub       h+11,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              mov       h+8,x                   'DO NOT MODIFY.
              mov       h+20,x                  'DO NOT MODIFY.
              mov       h+7,x                   'DO NOT MODIFY.
              mov       h+19,x                  'DO NOT MODIFY.
              sub       h+14,x                  'DO NOT MODIFY.
              mov       h+17,x                  'DO NOT MODIFY.
              mov       h+18,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              mov       h+15,x                  'DO NOT MODIFY.
              add       h+13,x                  'DO NOT MODIFY.
              mov       h+21,x                  'DO NOT MODIFY.
              sub       h+12,x                  'DO NOT MODIFY.
              sub       h+10,x                  'DO NOT MODIFY.
              mov       h+16,x                  'DO NOT MODIFY.
              mov       h+22,x                  'DO NOT MODIFY.
              sub       h+11,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              add       h+8,x                   'DO NOT MODIFY.
              mov       h+23,x                  'DO NOT MODIFY.
              sub       h+20,x                  'DO NOT MODIFY.
              mov       h+3,x                   'DO NOT MODIFY.
              mov       h+4,x                   'DO NOT MODIFY.
              sub       h+7,x                   'DO NOT MODIFY.
              mov       h+6,x                   'DO NOT MODIFY.
              mov       h+2,x                   'DO NOT MODIFY.
              add       h+9,x                   'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+15,x                  'DO NOT MODIFY.
              add       h+21,x                  'DO NOT MODIFY.
              mov       h+1,x                   'DO NOT MODIFY.
              add       h+16,x                  'DO NOT MODIFY.
              sub       h+22,x                  'DO NOT MODIFY.
              mov       h+0,x                   'DO NOT MODIFY.
              add       h+18,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              add       h+8,x                   'DO NOT MODIFY.
              sub       h+23,x                  'DO NOT MODIFY.
              add       h+20,x                  'DO NOT MODIFY.
              sub       h+13,x                  'DO NOT MODIFY.
              sub       h+10,x                  'DO NOT MODIFY.
              sub       h+4,x                   'DO NOT MODIFY.
              sub       h+7,x                   'DO NOT MODIFY.
              mov       h+24,x                  'DO NOT MODIFY.
              add       h+14,x                  'DO NOT MODIFY.
              add       h+17,x                  'DO NOT MODIFY.
              mov       h+5,x                   'DO NOT MODIFY.
              sub       h+9,x                   'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+15,x                  'DO NOT MODIFY.
              sub       h+21,x                  'DO NOT MODIFY.
              add       h+1,x                   'DO NOT MODIFY.
              add       h+16,x                  'DO NOT MODIFY.
              add       h+19,x                  'DO NOT MODIFY.
              sub       h+6,x                   'DO NOT MODIFY.
              sub       h+2,x                   'DO NOT MODIFY.
              sub       h+0,x                   'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              add       h+23,x                  'DO NOT MODIFY.
              mov       h+26,x                  'DO NOT MODIFY.
              add       h+20,x                  'DO NOT MODIFY.
              add       h+12,x                  'DO NOT MODIFY.
              add       h+4,x                   'DO NOT MODIFY.
              sub       h+7,x                   'DO NOT MODIFY.
              add       h+24,x                  'DO NOT MODIFY.
              add       h+14,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              add       h+13,x                  'DO NOT MODIFY.
              sub       h+3,x                   'DO NOT MODIFY.
              add       h+16,x                  'DO NOT MODIFY.
              sub       h+19,x                  'DO NOT MODIFY.
              sub       h+17,x                  'DO NOT MODIFY.
              sub       h+5,x                   'DO NOT MODIFY.
              mov       h+25,x                  'DO NOT MODIFY.
              add       h+9,x                   'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+15,x                  'DO NOT MODIFY.
              sub       h+20,x                  'DO NOT MODIFY.
              add       h+12,x                  'DO NOT MODIFY.
              sub       h+4,x                   'DO NOT MODIFY.
              add       h+6,x                   'DO NOT MODIFY.
              sub       h+14,x                  'DO NOT MODIFY.
              add       h+18,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+23,x                  'DO NOT MODIFY.
              add       h+1,x                   'DO NOT MODIFY.
              sub       h+7,x                   'DO NOT MODIFY.
              sub       h+16,x                  'DO NOT MODIFY.
              add       h+22,x                  'DO NOT MODIFY.
              sub       h+2,x                   'DO NOT MODIFY.
              sub       h+17,x                  'DO NOT MODIFY.
              add       h+25,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+15,x                  'DO NOT MODIFY.
              add       h+20,x                  'DO NOT MODIFY.
              sub       h+3,x                   'DO NOT MODIFY.
              add       h+19,x                  'DO NOT MODIFY.
              add       h+6,x                   'DO NOT MODIFY.
              sub       h+14,x                  'DO NOT MODIFY.
              add       h+5,x                   'DO NOT MODIFY.
              sub       h+11,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+8,x                   'DO NOT MODIFY.
              sub       h+23,x                  'DO NOT MODIFY.
              add       h+21,x                  'DO NOT MODIFY.
              sub       h+12,x                  'DO NOT MODIFY.
              add       h+10,x                  'DO NOT MODIFY.
              add       h+1,x                   'DO NOT MODIFY.
              add       h+24,x                  'DO NOT MODIFY.
              add       h+17,x                  'DO NOT MODIFY.
              sub       h+9,x                   'DO NOT MODIFY.
              add       h+18,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              sub       h+15,x                  'DO NOT MODIFY.
              sub       h+20,x                  'DO NOT MODIFY.
              add       h+4,x                   'DO NOT MODIFY.
              sub       h+16,x                  'DO NOT MODIFY.
              sub       h+14,x                  'DO NOT MODIFY.
              add       h+5,x                   'DO NOT MODIFY.
              add       h+0,x                   'DO NOT MODIFY.
              add       h+11,x                  'DO NOT MODIFY.
              sar       x,#1                    'DO NOT MODIFY.
              add       h+8,x                   'DO NOT MODIFY.
              add       h+23,x                  'DO NOT MODIFY.
              sub       h+26,x                  'DO NOT MODIFY.
              add       h+13,x                  'DO NOT MODIFY.
              add       h+3,x                   'DO NOT MODIFY.
              add       h+10,x                  'DO NOT MODIFY.
              add       h+1,x                   'DO NOT MODIFY.
              add       h+24,x                  'DO NOT MODIFY.
              sub       h+22,x                  'DO NOT MODIFY.
              sub       h+19,x                  'DO NOT MODIFY.
              add       h+6,x                   'DO NOT MODIFY.
              add       h+17,x                  'DO NOT MODIFY.
              add       h+25,x                  'DO NOT MODIFY.
              add       h+9,x                   'DO NOT MODIFY.
              sub       h+18,x                  'DO NOT MODIFY.

'Select the correct channel to process.

              test     sel,#1 wz                'DO NOT MODIFY.
        if_z  jmp      #b_stages                'DO NOT MODIFY.

'Apply input * coefficients to FIR stages and shift one step.

              mov       y,astage+0              'DO NOT MODIFY.

'Writing the y value here allows overlapped processing with another cog.
'Delete the next line to process the output further in this cog.

              call      #put_ya                 'Write the next output.

              add       astage+1,h+0            'DO NOT MODIFY.
              mov       astage+0,astage+1       'DO NOT MODIFY.
              mov       astage+1,astage+2       'DO NOT MODIFY.
              add       astage+3,h+1            'DO NOT MODIFY.
              mov       astage+2,astage+3       'DO NOT MODIFY.
              mov       astage+3,astage+4       'DO NOT MODIFY.
              add       astage+5,h+2            'DO NOT MODIFY.
              mov       astage+4,astage+5       'DO NOT MODIFY.
              mov       astage+5,astage+6       'DO NOT MODIFY.
              add       astage+7,h+3            'DO NOT MODIFY.
              mov       astage+6,astage+7       'DO NOT MODIFY.
              mov       astage+7,astage+8       'DO NOT MODIFY.
              add       astage+9,h+4            'DO NOT MODIFY.
              mov       astage+8,astage+9       'DO NOT MODIFY.
              mov       astage+9,astage+10      'DO NOT MODIFY.
              add       astage+11,h+5           'DO NOT MODIFY.
              mov       astage+10,astage+11     'DO NOT MODIFY.
              mov       astage+11,astage+12     'DO NOT MODIFY.
              sub       astage+13,h+6           'DO NOT MODIFY.
              mov       astage+12,astage+13     'DO NOT MODIFY.
              mov       astage+13,astage+14     'DO NOT MODIFY.
              sub       astage+15,h+7           'DO NOT MODIFY.
              mov       astage+14,astage+15     'DO NOT MODIFY.
              mov       astage+15,astage+16     'DO NOT MODIFY.
              sub       astage+17,h+8           'DO NOT MODIFY.
              mov       astage+16,astage+17     'DO NOT MODIFY.
              mov       astage+17,astage+18     'DO NOT MODIFY.
              sub       astage+19,h+9           'DO NOT MODIFY.
              mov       astage+18,astage+19     'DO NOT MODIFY.
              mov       astage+19,astage+20     'DO NOT MODIFY.
              sub       astage+21,h+10          'DO NOT MODIFY.
              mov       astage+20,astage+21     'DO NOT MODIFY.
              mov       astage+21,astage+22     'DO NOT MODIFY.
              sub       astage+23,h+11          'DO NOT MODIFY.
              mov       astage+22,astage+23     'DO NOT MODIFY.
              mov       astage+23,astage+24     'DO NOT MODIFY.
              sub       astage+25,h+12          'DO NOT MODIFY.
              mov       astage+24,astage+25     'DO NOT MODIFY.
              mov       astage+25,astage+26     'DO NOT MODIFY.
              add       astage+27,h+13          'DO NOT MODIFY.
              mov       astage+26,astage+27     'DO NOT MODIFY.
              mov       astage+27,astage+28     'DO NOT MODIFY.
              add       astage+29,h+14          'DO NOT MODIFY.
              mov       astage+28,astage+29     'DO NOT MODIFY.
              mov       astage+29,astage+30     'DO NOT MODIFY.
              add       astage+31,h+15          'DO NOT MODIFY.
              mov       astage+30,astage+31     'DO NOT MODIFY.
              mov       astage+31,astage+32     'DO NOT MODIFY.
              sub       astage+33,h+16          'DO NOT MODIFY.
              mov       astage+32,astage+33     'DO NOT MODIFY.
              mov       astage+33,astage+34     'DO NOT MODIFY.
              sub       astage+35,h+17          'DO NOT MODIFY.
              mov       astage+34,astage+35     'DO NOT MODIFY.
              mov       astage+35,astage+36     'DO NOT MODIFY.
              sub       astage+37,h+18          'DO NOT MODIFY.
              mov       astage+36,astage+37     'DO NOT MODIFY.
              mov       astage+37,astage+38     'DO NOT MODIFY.
              sub       astage+39,h+19          'DO NOT MODIFY.
              mov       astage+38,astage+39     'DO NOT MODIFY.
              mov       astage+39,astage+40     'DO NOT MODIFY.
              sub       astage+41,h+20          'DO NOT MODIFY.
              mov       astage+40,astage+41     'DO NOT MODIFY.
              mov       astage+41,astage+42     'DO NOT MODIFY.
              sub       astage+43,h+21          'DO NOT MODIFY.
              mov       astage+42,astage+43     'DO NOT MODIFY.
              mov       astage+43,astage+44     'DO NOT MODIFY.
              sub       astage+45,h+22          'DO NOT MODIFY.
              mov       astage+44,astage+45     'DO NOT MODIFY.
              mov       astage+45,astage+46     'DO NOT MODIFY.
              sub       astage+47,h+23          'DO NOT MODIFY.
              mov       astage+46,astage+47     'DO NOT MODIFY.
              mov       astage+47,astage+48     'DO NOT MODIFY.
              sub       astage+49,h+24          'DO NOT MODIFY.
              mov       astage+48,astage+49     'DO NOT MODIFY.
              mov       astage+49,astage+50     'DO NOT MODIFY.
              sub       astage+51,h+25          'DO NOT MODIFY.
              mov       astage+50,astage+51     'DO NOT MODIFY.
              mov       astage+51,astage+52     'DO NOT MODIFY.
              mov       astage+52,h+26          'DO NOT MODIFY.

'Add code here to further process y if the call to put_y was deleted above.          

              test      sel,#2 wz               'DO NOT MODIFY.
        if_z  jmp       #main_lp                'DO NOT MODIFY.
        
b_stages        
              add       bstage+0,h+26           'DO NOT MODIFY.
              mov       y,bstage+0              'DO NOT MODIFY.

'Writing the y value here allows overlapped processing with another cog.
'Delete the next line to process the output further in this cog.

              call      #put_yb                 'Write the next output.

              mov       bstage+0,bstage+1       'DO NOT MODIFY.
              sub       bstage+2,h+25           'DO NOT MODIFY.
              mov       bstage+1,bstage+2       'DO NOT MODIFY.
              mov       bstage+2,bstage+3       'DO NOT MODIFY.
              sub       bstage+4,h+24           'DO NOT MODIFY.
              mov       bstage+3,bstage+4       'DO NOT MODIFY.
              mov       bstage+4,bstage+5       'DO NOT MODIFY.
              sub       bstage+6,h+23           'DO NOT MODIFY.
              mov       bstage+5,bstage+6       'DO NOT MODIFY.
              mov       bstage+6,bstage+7       'DO NOT MODIFY.
              sub       bstage+8,h+22           'DO NOT MODIFY.
              mov       bstage+7,bstage+8       'DO NOT MODIFY.
              mov       bstage+8,bstage+9       'DO NOT MODIFY.
              sub       bstage+10,h+21          'DO NOT MODIFY.
              mov       bstage+9,bstage+10      'DO NOT MODIFY.
              mov       bstage+10,bstage+11     'DO NOT MODIFY.
              sub       bstage+12,h+20          'DO NOT MODIFY.
              mov       bstage+11,bstage+12     'DO NOT MODIFY.
              mov       bstage+12,bstage+13     'DO NOT MODIFY.
              sub       bstage+14,h+19          'DO NOT MODIFY.
              mov       bstage+13,bstage+14     'DO NOT MODIFY.
              mov       bstage+14,bstage+15     'DO NOT MODIFY.
              sub       bstage+16,h+18          'DO NOT MODIFY.
              mov       bstage+15,bstage+16     'DO NOT MODIFY.
              mov       bstage+16,bstage+17     'DO NOT MODIFY.
              sub       bstage+18,h+17          'DO NOT MODIFY.
              mov       bstage+17,bstage+18     'DO NOT MODIFY.
              mov       bstage+18,bstage+19     'DO NOT MODIFY.
              sub       bstage+20,h+16          'DO NOT MODIFY.
              mov       bstage+19,bstage+20     'DO NOT MODIFY.
              mov       bstage+20,bstage+21     'DO NOT MODIFY.
              add       bstage+22,h+15          'DO NOT MODIFY.
              mov       bstage+21,bstage+22     'DO NOT MODIFY.
              mov       bstage+22,bstage+23     'DO NOT MODIFY.
              add       bstage+24,h+14          'DO NOT MODIFY.
              mov       bstage+23,bstage+24     'DO NOT MODIFY.
              mov       bstage+24,bstage+25     'DO NOT MODIFY.
              add       bstage+26,h+13          'DO NOT MODIFY.
              mov       bstage+25,bstage+26     'DO NOT MODIFY.
              mov       bstage+26,bstage+27     'DO NOT MODIFY.
              sub       bstage+28,h+12          'DO NOT MODIFY.
              mov       bstage+27,bstage+28     'DO NOT MODIFY.
              mov       bstage+28,bstage+29     'DO NOT MODIFY.
              sub       bstage+30,h+11          'DO NOT MODIFY.
              mov       bstage+29,bstage+30     'DO NOT MODIFY.
              mov       bstage+30,bstage+31     'DO NOT MODIFY.
              sub       bstage+32,h+10          'DO NOT MODIFY.
              mov       bstage+31,bstage+32     'DO NOT MODIFY.
              mov       bstage+32,bstage+33     'DO NOT MODIFY.
              sub       bstage+34,h+9           'DO NOT MODIFY.
              mov       bstage+33,bstage+34     'DO NOT MODIFY.
              mov       bstage+34,bstage+35     'DO NOT MODIFY.
              sub       bstage+36,h+8           'DO NOT MODIFY.
              mov       bstage+35,bstage+36     'DO NOT MODIFY.
              mov       bstage+36,bstage+37     'DO NOT MODIFY.
              sub       bstage+38,h+7           'DO NOT MODIFY.
              mov       bstage+37,bstage+38     'DO NOT MODIFY.
              mov       bstage+38,bstage+39     'DO NOT MODIFY.
              sub       bstage+40,h+6           'DO NOT MODIFY.
              mov       bstage+39,bstage+40     'DO NOT MODIFY.
              mov       bstage+40,bstage+41     'DO NOT MODIFY.
              add       bstage+42,h+5           'DO NOT MODIFY.
              mov       bstage+41,bstage+42     'DO NOT MODIFY.
              mov       bstage+42,bstage+43     'DO NOT MODIFY.
              add       bstage+44,h+4           'DO NOT MODIFY.
              mov       bstage+43,bstage+44     'DO NOT MODIFY.
              mov       bstage+44,bstage+45     'DO NOT MODIFY.
              add       bstage+46,h+3           'DO NOT MODIFY.
              mov       bstage+45,bstage+46     'DO NOT MODIFY.
              mov       bstage+46,bstage+47     'DO NOT MODIFY.
              add       bstage+48,h+2           'DO NOT MODIFY.
              mov       bstage+47,bstage+48     'DO NOT MODIFY.
              mov       bstage+48,bstage+49     'DO NOT MODIFY.
              add       bstage+50,h+1           'DO NOT MODIFY.
              mov       bstage+49,bstage+50     'DO NOT MODIFY.
              mov       bstage+50,bstage+51     'DO NOT MODIFY.
              add       bstage+52,h+0           'DO NOT MODIFY.
              mov       bstage+51,bstage+52     'DO NOT MODIFY.
              mov       bstage+52,bstage+53     'DO NOT MODIFY.

'Add code here to further process y if the call to put_y was deleted above.          

              jmp       #main_lp                'DO NOT MODIFY.

'This subroutine acquires the next input sample (x) from the hub.
'It can be changed to acquire the sample in another way. The value
'returned in sel should reflect which channel(s) to process:
'bit 0 set for channel a; bit 1 set for channel b.

get_x         rdlong    sel,par wz              'Read select/busy flag. Is it zero?
       if_z   jmp       #get_x                  '  Yes: Keep checking.
       
              rdlong    x,xy_addr               '  No:  Read the input.
get_x_ret     ret

'This subroutine writes the output of the filter (y) to the hub.
'It can be changed to deal with the output in another way.

put_ya        wrlong    y,xy_addr               'Write result to hub.
              test      sel,#2 wz
        if_z  wrlong    zero,par                'Signal completion.
put_ya_ret    ret

put_yb        wrlong    y,yb_addr
              wrlong    zero,par
put_yb_ret    ret

'Constants and variables.

astage        long      0[53]                   'DO NOT MODIFY.
bstage        long      0[53]                   'DO NOT MODIFY.
zero          long      0

xy_addr       res       1                       'Hub address of argument.
yb_addr       res       1
sel           res       1
x             res       1                       'DO NOT MODIFY.
y             res       1                       'DO NOT MODIFY.
h             res       27                      'DO NOT MODIFY.

''=======[ License ]==========================================================
{{{
***************************************************************************************
*                                                                                     *
* Permission is hereby granted, free of charge, to any person obtaining a copy of     *
* this software and associated documentation files (the "Software"), to deal in       *
* the Software without restriction, including without limitation the rights to use,   *
* copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the     *
* Software, and to permit persons to whom the Software is furnished to do so,         *
* subject to the following conditions:                                                *
*                                                                                     *
* The above copyright notice and this permission notice shall be included in all      *
* copies or substantial portions of the Software.                                     *
*                                                                                     *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, *
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A       *
* PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT  *
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION   *
* OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                              *
*                                                                                     *
***************************************************************************************
}}